@@ -3,6 +3,7 @@
  This file is part of the KDE project.
 
 Copyright (C) 2015 Martin Gräßlin <mgraesslin@kde.org>
+Copyright (C) 2019 Vlad Zahorodnii <vlad.zahorodnii@kde.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -45,6 +46,7 @@ class Decoration;
 
 namespace KWin
 {
+class Group;
 
 namespace TabBox
 {
@@ -221,7 +223,7 @@ class KWIN_EXPORT AbstractClient : public Toplevel
      * The geometry of this Client. Be aware that depending on resize mode the geometryChanged signal
      * might be emitted at each resize step or only at the end of the resize operation.
      */
-    Q_PROPERTY(QRect geometry READ geometry WRITE setGeometry)
+    Q_PROPERTY(QRect geometry READ frameGeometry WRITE setFrameGeometry)
 
     /**
      * Whether the Client is currently being moved by the user.
@@ -384,6 +386,7 @@ public:
 
     bool wantsTabFocus() const;
 
+    QMargins frameMargins() const override;
     QPoint clientPos() const override {
         return QPoint(borderLeft(), borderTop());
     }
@@ -493,7 +496,7 @@ public:
     /**
      * The maximise mode requested by the server.
      * For X this always matches maximizeMode, for wayland clients it
-     * is asyncronous
+     * is asynchronous
      */
     virtual MaximizeMode requestedMaximizeMode() const;
     void maximize(MaximizeMode);
@@ -622,15 +625,15 @@ public:
     void updateLayer();
 
     enum ForceGeometry_t { NormalGeometrySet, ForceGeometrySet };
-    void move(int x, int y, ForceGeometry_t force = NormalGeometrySet);
+    virtual void move(int x, int y, ForceGeometry_t force = NormalGeometrySet);
     void move(const QPoint &p, ForceGeometry_t force = NormalGeometrySet);
     virtual void resizeWithChecks(int w, int h, ForceGeometry_t force = NormalGeometrySet) = 0;
     void resizeWithChecks(const QSize& s, ForceGeometry_t force = NormalGeometrySet);
     void keepInArea(QRect area, bool partial = false);
     virtual QSize minSize() const;
     virtual QSize maxSize() const;
-    virtual void setGeometry(int x, int y, int w, int h, ForceGeometry_t force = NormalGeometrySet) = 0;
-    void setGeometry(const QRect& r, ForceGeometry_t force = NormalGeometrySet);
+    virtual void setFrameGeometry(int x, int y, int w, int h, ForceGeometry_t force = NormalGeometrySet) = 0;
+    void setFrameGeometry(const QRect &rect, ForceGeometry_t force = NormalGeometrySet);
     /// How to resize the window in order to obey constains (mainly aspect ratios)
     enum Sizemode {
         SizemodeAny,
@@ -653,6 +656,43 @@ public:
     QSize adjustedSize(const QSize&, Sizemode mode = SizemodeAny) const;
     QSize adjustedSize() const;
 
+    /**
+     * Calculates the matching client position for the given frame position @p point.
+     */
+    virtual QPoint framePosToClientPos(const QPoint &point) const;
+    /**
+     * Calculates the matching frame position for the given client position @p point.
+     */
+    virtual QPoint clientPosToFramePos(const QPoint &point) const;
+    /**
+     * Calculates the matching client size for the given frame size @p size.
+     *
+     * Notice that size constraints won't be applied.
+     *
+     * Default implementation returns the frame size with frame margins being excluded.
+     */
+    virtual QSize frameSizeToClientSize(const QSize &size) const;
+    /**
+     * Calculates the matching frame size for the given client size @p size.
+     *
+     * Notice that size constraints won't be applied.
+     *
+     * Default implementation returns the client size with frame margins being included.
+     */
+    virtual QSize clientSizeToFrameSize(const QSize &size) const;
+    /**
+     * Calculates the matching client rect for the given frame rect @p rect.
+     *
+     * Notice that size constraints won't be applied.
+     */
+    QRect frameRectToClientRect(const QRect &rect) const;
+    /**
+     * Calculates the matching frame rect for the given client rect @p rect.
+     *
+     * Notice that size constraints won't be applied.
+     */
+    QRect clientRectToFrameRect(const QRect &rect) const;
+
     bool isMove() const {
         return isMoveResize() && moveResizePointerMode() == PositionCenter;
     }
@@ -998,9 +1038,8 @@ protected:
     };
     PendingGeometry_t pendingGeometryUpdate() const;
     void setPendingGeometryUpdate(PendingGeometry_t update);
-    QRect geometryBeforeUpdateBlocking() const {
-        return m_geometryBeforeUpdateBlocking;
-    }
+    QRect bufferGeometryBeforeUpdateBlocking() const;
+    QRect frameGeometryBeforeUpdateBlocking() const;
     void updateGeometryBeforeUpdateBlocking();
     /**
      * Schedules a repaint for the visibleRect before and after a
@@ -1218,7 +1257,8 @@ private:
     PendingGeometry_t m_pendingGeometryUpdate = PendingGeometryNone;
     friend class GeometryUpdatesBlocker;
     QRect m_visibleRectBeforeGeometryUpdate;
-    QRect m_geometryBeforeUpdateBlocking;
+    QRect m_bufferGeometryBeforeUpdateBlocking;
+    QRect m_frameGeometryBeforeUpdateBlocking;
     QRect m_virtualKeyboardGeometry;
     QRect m_keyboardGeometryRestore;
 
@@ -1284,9 +1324,9 @@ inline void AbstractClient::resizeWithChecks(const QSize& s, AbstractClient::For
     resizeWithChecks(s.width(), s.height(), force);
 }
 
-inline void AbstractClient::setGeometry(const QRect& r, ForceGeometry_t force)
+inline void AbstractClient::setFrameGeometry(const QRect &rect, ForceGeometry_t force)
 {
-    setGeometry(r.x(), r.y(), r.width(), r.height(), force);
+    setFrameGeometry(rect.x(), rect.y(), rect.width(), rect.height(), force);
 }
 
 inline const QList<AbstractClient*>& AbstractClient::transients() const
