@@ -45,7 +45,7 @@ Q_DECLARE_METATYPE(KWin::ElectricBorder)
 
 /**
  * This test verifies the NoGlobalShortcuts initialization flag
- **/
+ */
 class NoGlobalShortcutsTest : public QObject
 {
     Q_OBJECT
@@ -70,7 +70,7 @@ class Target : public QObject
 
 public:
     Target();
-    virtual ~Target();
+    ~Target() override;
 
 public Q_SLOTS:
     Q_SCRIPTABLE void shortcut();
