@@ -524,6 +524,8 @@ QByteArray GLPlatform::driverToString8(Driver driver)
         return QByteArrayLiteral("VMware (SVGA3D)");
     case Driver_Qualcomm:
         return QByteArrayLiteral("Qualcomm");
+    case Driver_Virgl:
+        return QByteArrayLiteral("Virgl (virtio-gpu, Qemu/KVM guest)");
 
     default:
         return QByteArrayLiteral("Unknown");
@@ -680,7 +682,7 @@ void GLPlatform::detect(OpenGLPlatformInterface platformInterface)
     // Parse the Mesa version
     const int mesaIndex = versionTokens.indexOf("Mesa");
     if (mesaIndex != -1) {
-        const QByteArray version = versionTokens.at(mesaIndex + 1);
+        const QByteArray &version = versionTokens.at(mesaIndex + 1);
         m_mesaVersion = parseVersionString(version);
     }
 
@@ -718,7 +720,7 @@ void GLPlatform::detect(OpenGLPlatformInterface platformInterface)
     if (m_renderer.startsWith("Mesa DRI R")) {
         // Sample renderer string: Mesa DRI R600 (RV740 94B3) 20090101 x86/MMX/SSE2 TCL DRI2
         const QList<QByteArray> tokens = m_renderer.split(' ');
-        const QByteArray chipClass = tokens.at(2);
+        const QByteArray &chipClass = tokens.at(2);
         m_chipset = tokens.at(3).mid(1, -1); // Strip the leading '('
 
         if (chipClass == "R100")
@@ -899,6 +901,11 @@ void GLPlatform::detect(OpenGLPlatformInterface platformInterface)
         else if (m_vendor == "VMware, Inc." && m_chipset.contains("SVGA3D")) {
             m_driver = Driver_VMware;
         }
+
+        // virgl
+        else if (m_renderer == "virgl") {
+            m_driver = Driver_Virgl;
+        }
     }
 
     // Driver/GPU specific features
@@ -1017,6 +1024,11 @@ void GLPlatform::detect(OpenGLPlatformInterface platformInterface)
         m_recommendedCompositor = OpenGL2Compositing;
     }
 
+    if (m_driver == Driver_Virgl) {
+        m_virtualMachine = true;
+        m_recommendedCompositor = OpenGL2Compositing;
+    }
+
     // and force back to shader supported on gles, we wouldn't have got a context if not supported
     if (isGLES()) {
         m_supportsGLSL = true;
@@ -1171,6 +1183,11 @@ bool GLPlatform::isVMware() const
     return m_driver == Driver_VMware;
 }
 
+bool GLPlatform::isVirgl() const
+{
+    return m_driver == Driver_Virgl;
+}
+
 bool GLPlatform::isSoftwareEmulation() const
 {
     return m_driver == Driver_Softpipe || m_driver == Driver_Swrast || m_driver == Driver_Llvmpipe;
