@@ -42,7 +42,7 @@ enum GLFeature {
      * Set when a texture bound to a pixmap uses the same storage as the pixmap,
      * and thus doesn't need to be rebound when the contents of the pixmap
      * has changed.
-     **/
+     */
     LooseBinding,
 
     /**
@@ -51,7 +51,7 @@ enum GLFeature {
      * - GL_ARB_fragment_shader
      * - GL_ARB_vertex_shader
      * - GL_ARB_shading_language_100
-     **/
+     */
     GLSL,
 
     /**
@@ -64,12 +64,12 @@ enum GLFeature {
      * - No texture fetches in vertex shaders
      * - Max 32 texture fetches in fragment shaders
      * - Max 4 texture indirections
-     **/
+     */
     LimitedGLSL,
 
     /**
      * Set when the driver supports GL_ARB_texture_non_power_of_two.
-     **/
+     */
     TextureNPOT,
 
     /**
@@ -81,7 +81,7 @@ enum GLFeature {
      * - GL_CLAMP
      * - GL_CLAMP_TO_EDGE
      * - GL_CLAMP_TO_BORDER
-     **/
+     */
     LimitedNPOT
 };
 
@@ -103,6 +103,7 @@ enum Driver {
     Driver_VMware,
     Driver_Qualcomm,
     Driver_RadeonSI,
+    Driver_Virgl,
     Driver_Unknown
 };
 
@@ -162,42 +163,42 @@ public:
 
     /**
      * Runs the detection code using the current OpenGL context.
-     **/
+     */
     void detect(OpenGLPlatformInterface platformInterface);
 
     /**
      * Prints the results of the detection code.
-     **/
+     */
     void printResults() const;
 
     /**
      * Returns a pointer to the GLPlatform instance.
-     **/
+     */
     static GLPlatform *instance();
 
     /**
      * Returns true if the driver support the given feature, and false otherwise.
-     **/
+     */
     bool supports(GLFeature feature) const;
 
     /**
      * Returns the OpenGL version.
-     **/
+     */
     qint64 glVersion() const;
 
     /**
      * Returns the GLSL version if the driver supports GLSL, and 0 otherwise.
-     **/
+     */
     qint64 glslVersion() const;
 
     /**
      * Returns the Mesa version if the driver is a Mesa driver, and 0 otherwise.
-     **/
+     */
     qint64 mesaVersion() const;
 
     /**
      * Returns the Gallium version if the driver is a Gallium driver, and 0 otherwise.
-     **/
+     */
     qint64 galliumVersion() const;
 
     /**
@@ -207,123 +208,129 @@ public:
      * following the doupling of the X server from the katamari.
      *
      * For non X.org servers, this method returns 0.
-     **/
+     */
     qint64 serverVersion() const;
 
     /**
      * Returns the Linux kernel version.
      *
      * If the kernel is not a Linux kernel, this method returns 0.
-     **/
+     */
     qint64 kernelVersion() const;
 
     /**
      * Returns the driver version.
      *
      * For Mesa drivers, this is the same as the Mesa version number.
-     **/
+     */
     qint64 driverVersion() const;
 
     /**
      * Returns the driver.
-     **/
+     */
     Driver driver() const;
 
     /**
      * Returns the chip class.
-     **/
+     */
     ChipClass chipClass() const;
 
     /**
      * Returns true if the driver is a Mesa driver, and false otherwise.
-     **/
+     */
     bool isMesaDriver() const;
 
     /**
      * Returns true if the driver is a Gallium driver, and false otherwise.
-     **/
+     */
     bool isGalliumDriver() const;
 
     /**
      * Returns true if the GPU is a Radeon GPU, and false otherwise.
-     **/
+     */
     bool isRadeon() const;
 
     /**
      * Returns true if the GPU is an NVIDIA GPU, and false otherwise.
-     **/
+     */
     bool isNvidia() const;
 
     /**
      * Returns true if the GPU is an Intel GPU, and false otherwise.
-     **/
+     */
     bool isIntel() const;
 
     /**
      * @returns @c true if the "GPU" is a VirtualBox GPU, and @c false otherwise.
      * @since 4.10
-     **/
+     */
     bool isVirtualBox() const;
 
     /**
      * @returns @c true if the "GPU" is a VMWare GPU, and @c false otherwise.
      * @since 4.10
-     **/
+     */
     bool isVMware() const;
 
     /**
      * @returns @c true if OpenGL is emulated in software.
      * @since 4.7
-     **/
+     */
     bool isSoftwareEmulation() const;
 
     /**
      * @returns @c true if the driver is known to be from a virtual machine.
      * @since 4.10
-     **/
+     */
     bool isVirtualMachine() const;
 
     /**
      * @returns @c true if the GPU is a Qualcomm Adreno GPU, and false otherwise
      * @since 5.8
-     **/
+     */
     bool isAdreno() const;
 
+    /**
+     * @returns @c true if the "GPU" is a virtio-gpu (Qemu/KVM)
+     * @since 5.18
+     **/
+    bool isVirgl() const;
+
     /**
      * @returns the GL_VERSION string as provided by the driver.
      * @since 4.9
-     **/
+     */
     const QByteArray &glVersionString() const;
     /**
      * @returns the GL_RENDERER string as provided by the driver.
      * @since 4.9
-     **/
+     */
     const QByteArray &glRendererString() const;
     /**
      * @returns the GL_VENDOR string as provided by the driver.
      * @since 4.9
-     **/
+     */
     const QByteArray &glVendorString() const;
     /**
      * @returns the GL_SHADING_LANGUAGE_VERSION string as provided by the driver.
      * If the driver does not support the OpenGL Shading Language a null bytearray is returned.
      * @since 4.9
-     **/
+     */
     const QByteArray &glShadingLanguageVersionString() const;
     /**
      * @returns Whether the driver supports loose texture binding.
      * @since 4.9
-     **/
+     */
     bool isLooseBinding() const;
     /**
      * @returns Whether OpenGL ES is used
-     **/
+     */
     bool isGLES() const;
 
     /**
      * @returns The CompositingType recommended by the driver.
      * @since 4.10
-     **/
+     */
     CompositingType recommendedCompositor() const;
 
     /**
@@ -331,13 +338,13 @@ public:
      * when updating an unused range of a buffer object, and false otherwise.
      *
      * @since 4.11
-     **/
+     */
     bool preferBufferSubData() const;
 
     /**
      * @returns The OpenGLPlatformInterface currently used
      * @since 5.0
-     **/
+     */
     OpenGLPlatformInterface platformInterface() const;
 
     /**
@@ -350,7 +357,7 @@ public:
      * @see galliumVersion
      * @see kernelVersion
      * @see serverVersion
-     **/
+     */
     static QString versionToString(qint64 version);
     /**
      * @returns a human readable form of the @p version as a QByteArray.
@@ -362,33 +369,33 @@ public:
      * @see galliumVersion
      * @see kernelVersion
      * @see serverVersion
-     **/
+     */
     static QByteArray versionToString8(qint64 version);
 
     /**
      * @returns a human readable form for the @p driver as a QString.
      * @since 4.9
      * @see driver
-     **/
+     */
     static QString driverToString(Driver driver);
     /**
      * @returns a human readable form for the @p driver as a QByteArray.
      * @since 5.5
      * @see driver
-     **/
+     */
     static QByteArray driverToString8(Driver driver);
 
     /**
      * @returns a human readable form for the @p chipClass as a QString.
      * @since 4.9
      * @see chipClass
-     **/
+     */
     static QString chipClassToString(ChipClass chipClass);
     /**
      * @returns a human readable form for the @p chipClass as a QByteArray.
      * @since 5.5
      * @see chipClass
-     **/
+     */
     static QByteArray chipClassToString8(ChipClass chipClass);
 
 private:
