@@ -18,27 +18,30 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "x11windowed_backend.h"
+#include "x11windowed_output.h"
 #include "scene_qpainter_x11_backend.h"
 #include "logging.h"
 #include "wayland_server.h"
 #include "xcbutils.h"
 #include "egl_x11_backend.h"
-#include "screens.h"
+#include "outputscreens.h"
 #include <kwinxrenderutils.h>
 // KDE
 #include <KLocalizedString>
-#include <NETWM>
 #include <QAbstractEventDispatcher>
 #include <QCoreApplication>
-#include <QIcon>
 #include <QSocketNotifier>
 // kwayland
-#include <KWayland/Server/buffer_interface.h>
 #include <KWayland/Server/display.h>
 #include <KWayland/Server/seat_interface.h>
-#include <KWayland/Server/surface_interface.h>
 // xcb
 #include <xcb/xcb_keysyms.h>
+// X11
+#if HAVE_X11_XINPUT
+#include "ge_event_mem_mover.h"
+#include <X11/extensions/XInput2.h>
+#include <X11/extensions/XI2proto.h>
+#endif
 // system
 #include <linux/input.h>
 #include <X11/Xlib-xcb.h>
@@ -59,11 +62,6 @@ X11WindowedBackend::~X11WindowedBackend()
         if (m_keySymbols) {
             xcb_key_symbols_free(m_keySymbols);
         }
-        for (auto it = m_windows.begin(); it != m_windows.end(); ++it) {
-            xcb_unmap_window(m_connection, (*it).window);
-            xcb_destroy_window(m_connection, (*it).window);
-            delete (*it).winInfo;
-        }
         if (m_cursor) {
             xcb_free_cursor(m_connection, m_cursor);
         }
@@ -92,8 +90,9 @@ void X11WindowedBackend::init()
                 m_screen = it.data;
             }
         }
+        initXInput();
         XRenderUtils::init(m_connection, m_screen->root);
-        createWindow();
+        createOutputs();
         connect(kwinApp(), &Application::workspaceCreated, this, &X11WindowedBackend::startEventReading);
         connect(this, &X11WindowedBackend::cursorChanged, this,
             [this] {
@@ -103,67 +102,85 @@ void X11WindowedBackend::init()
         setReady(true);
         waylandServer()->seat()->setHasPointer(true);
         waylandServer()->seat()->setHasKeyboard(true);
+        if (m_hasXInput) {
+            waylandServer()->seat()->setHasTouch(true);
+        }
         emit screensQueried();
     } else {
         emit initFailed();
     }
 }
 
-void X11WindowedBackend::createWindow()
+void X11WindowedBackend::initXInput()
+{
+#if HAVE_X11_XINPUT
+    int xi_opcode, event, error;
+    // init XInput extension
+    if (!XQueryExtension(m_display, "XInputExtension", &xi_opcode, &event, &error)) {
+        qCDebug(KWIN_X11WINDOWED) << "XInputExtension not present";
+        return;
+    }
+
+    // verify that the XInput extension is at at least version 2.0
+    int major = 2, minor = 2;
+    int result = XIQueryVersion(m_display, &major, &minor);
+    if (result != Success) {
+        qCDebug(KWIN_X11WINDOWED) << "Failed to init XInput 2.2, trying 2.0";
+        minor = 0;
+        if (XIQueryVersion(m_display, &major, &minor) != Success) {
+            qCDebug(KWIN_X11WINDOWED) << "Failed to init XInput";
+            return;
+        }
+    }
+    m_xiOpcode = xi_opcode;
+    m_majorVersion = major;
+    m_minorVersion = minor;
+    m_hasXInput = m_majorVersion >=2 && m_minorVersion >= 2;
+#endif
+}
+
+X11WindowedOutput *X11WindowedBackend::findOutput(xcb_window_t window) const
+{
+    auto it = std::find_if(m_outputs.constBegin(), m_outputs.constEnd(),
+        [window] (X11WindowedOutput *output) {
+            return output->window() == window;
+        }
+    );
+    if (it != m_outputs.constEnd()) {
+        return *it;
+    }
+    return nullptr;
+}
+
+void X11WindowedBackend::createOutputs()
 {
     Xcb::Atom protocolsAtom(QByteArrayLiteral("WM_PROTOCOLS"), false, m_connection);
     Xcb::Atom deleteWindowAtom(QByteArrayLiteral("WM_DELETE_WINDOW"), false, m_connection);
+
+    // we need to multiply the initial window size with the scale in order to
+    // create an output window of this size in the end
+    const int pixelWidth = initialWindowSize().width() * initialOutputScale() + 0.5;
+    const int pixelHeight = initialWindowSize().height() * initialOutputScale() + 0.5;
+    const int logicalWidth = initialWindowSize().width();
+
+    int logicalWidthSum = 0;
     for (int i = 0; i < initialOutputCount(); ++i) {
-        Output o;
-        o.window = xcb_generate_id(m_connection);
-        uint32_t mask = XCB_CW_BACK_PIXEL | XCB_CW_EVENT_MASK;
-        const uint32_t values[] = {
-            m_screen->black_pixel,
-            XCB_EVENT_MASK_KEY_PRESS |
-            XCB_EVENT_MASK_KEY_RELEASE |
-            XCB_EVENT_MASK_BUTTON_PRESS |
-            XCB_EVENT_MASK_BUTTON_RELEASE |
-            XCB_EVENT_MASK_POINTER_MOTION |
-            XCB_EVENT_MASK_ENTER_WINDOW |
-            XCB_EVENT_MASK_LEAVE_WINDOW |
-            XCB_EVENT_MASK_STRUCTURE_NOTIFY |
-            XCB_EVENT_MASK_EXPOSURE
-        };
-        o.scale = initialOutputScale();
-        o.size = initialWindowSize() * o.scale;
-        if (!m_windows.isEmpty()) {
-            const auto &p = m_windows.last();
-            o.internalPosition = QPoint(p.internalPosition.x() + p.size.width() / p.scale, 0);
-        }
-        xcb_create_window(m_connection, XCB_COPY_FROM_PARENT, o.window, m_screen->root,
-                        0, 0, o.size.width(), o.size.height(),
-                        0, XCB_WINDOW_CLASS_INPUT_OUTPUT, XCB_COPY_FROM_PARENT, mask, values);
-
-        o.winInfo = new NETWinInfo(m_connection, o.window, m_screen->root, NET::WMWindowType, NET::Properties2());
-        o.winInfo->setWindowType(NET::Normal);
-        o.winInfo->setPid(QCoreApplication::applicationPid());
-        QIcon windowIcon = QIcon::fromTheme(QStringLiteral("kwin"));
-        auto addIcon = [&o, &windowIcon] (const QSize &size) {
-            if (windowIcon.actualSize(size) != size) {
-                return;
-            }
-            NETIcon icon;
-            icon.data = windowIcon.pixmap(size).toImage().bits();
-            icon.size.width = size.width();
-            icon.size.height = size.height();
-            o.winInfo->setIcon(icon, false);
-        };
-        addIcon(QSize(16, 16));
-        addIcon(QSize(32, 32));
-        addIcon(QSize(48, 48));
-
-        xcb_map_window(m_connection, o.window);
+        auto *output = new X11WindowedOutput(this);
+        output->init(QPoint(logicalWidthSum, 0), QSize(pixelWidth, pixelHeight));
 
         m_protocols = protocolsAtom;
         m_deleteWindowProtocol = deleteWindowAtom;
-        xcb_change_property(m_connection, XCB_PROP_MODE_REPLACE, o.window, m_protocols, XCB_ATOM_ATOM, 32, 1, &m_deleteWindowProtocol);
 
-        m_windows << o;
+        xcb_change_property(m_connection,
+                            XCB_PROP_MODE_REPLACE,
+                            output->window(),
+                            m_protocols,
+                            XCB_ATOM_ATOM,
+                            32, 1,
+                            &m_deleteWindowProtocol);
+
+        logicalWidthSum += logicalWidth;
+        m_outputs << output;
     }
 
     updateWindowTitle();
@@ -186,6 +203,14 @@ void X11WindowedBackend::startEventReading()
     connect(QCoreApplication::eventDispatcher(), &QAbstractEventDispatcher::awake, this, processXcbEvents);
 }
 
+#if HAVE_X11_XINPUT
+
+static inline qreal fixed1616ToReal(FP1616 val)
+{
+    return (val) * 1.0 / (1 << 16);
+}
+#endif
+
 void X11WindowedBackend::handleEvent(xcb_generic_event_t *e)
 {
     const uint8_t eventType = e->response_type & ~0x80;
@@ -196,15 +221,12 @@ void X11WindowedBackend::handleEvent(xcb_generic_event_t *e)
         break;
     case XCB_MOTION_NOTIFY: {
             auto event = reinterpret_cast<xcb_motion_notify_event_t*>(e);
-            auto it = std::find_if(m_windows.constBegin(), m_windows.constEnd(), [event] (const Output &o) { return o.window == event->event; });
-            if (it == m_windows.constEnd()) {
+            const X11WindowedOutput *output = findOutput(event->event);
+            if (!output) {
                 break;
             }
-            //generally we don't need to normalise input to the output scale; however because we're getting input
-            //from a host window that doesn't understand scaling, we need to apply it ourselves so the cursor matches
-            pointerMotion(QPointF(event->root_x - (*it).xPosition.x() + (*it).internalPosition.x(),
-                                    event->root_y - (*it).xPosition.y() + (*it).internalPosition.y()) / it->scale,
-                                    event->time);
+            const QPointF position = output->mapFromGlobal(QPointF(event->root_x, event->root_y));
+            pointerMotion(position, event->time);
         }
         break;
     case XCB_KEY_PRESS:
@@ -229,13 +251,12 @@ void X11WindowedBackend::handleEvent(xcb_generic_event_t *e)
         break;
     case XCB_ENTER_NOTIFY: {
             auto event = reinterpret_cast<xcb_enter_notify_event_t*>(e);
-            auto it = std::find_if(m_windows.constBegin(), m_windows.constEnd(), [event] (const Output &o) { return o.window == event->event; });
-            if (it == m_windows.constEnd()) {
+            const X11WindowedOutput *output = findOutput(event->event);
+            if (!output) {
                 break;
             }
-            pointerMotion(QPointF(event->root_x - (*it).xPosition.x() + (*it).internalPosition.x(),
-                                    event->root_y - (*it).xPosition.y() + (*it).internalPosition.y()) / it->scale,
-                                    event->time);
+            const QPointF position = output->mapFromGlobal(QPointF(event->root_x, event->root_y));
+            pointerMotion(position, event->time);
         }
         break;
     case XCB_CLIENT_MESSAGE:
@@ -249,6 +270,43 @@ void X11WindowedBackend::handleEvent(xcb_generic_event_t *e)
             xcb_refresh_keyboard_mapping(m_keySymbols, reinterpret_cast<xcb_mapping_notify_event_t*>(e));
         }
         break;
+#if HAVE_X11_XINPUT
+    case XCB_GE_GENERIC: {
+        GeEventMemMover ge(e);
+        auto te = reinterpret_cast<xXIDeviceEvent*>(e);
+        const X11WindowedOutput *output = findOutput(te->event);
+        if (!output) {
+            break;
+        }
+
+        const QPointF position = output->mapFromGlobal(QPointF(fixed1616ToReal(te->root_x), fixed1616ToReal(te->root_y)));
+
+        switch (ge->event_type) {
+
+        case XI_TouchBegin: {
+            touchDown(te->detail, position, te->time);
+            touchFrame();
+            break;
+        }
+        case XI_TouchUpdate: {
+            touchMotion(te->detail, position, te->time);
+            touchFrame();
+            break;
+        }
+        case XI_TouchEnd: {
+            touchUp(te->detail, te->time);
+            touchFrame();
+            break;
+        }
+        case XI_TouchOwnership: {
+            auto te = reinterpret_cast<xXITouchOwnershipEvent*>(e);
+            XIAllowTouchEvents(m_display, te->deviceid, te->sourceid, te->touchid, XIAcceptTouch);
+            break;
+        }
+        }
+        break;
+    }
+#endif
     default:
         break;
     }
@@ -295,37 +353,39 @@ void X11WindowedBackend::updateWindowTitle()
     const QString title = QStringLiteral("%1 (%2) - %3").arg(i18n("KDE Wayland Compositor"))
                                                         .arg(waylandServer()->display()->socketName())
                                                         .arg(grab);
-    for (auto it = m_windows.constBegin(); it != m_windows.constEnd(); ++it) {
-        (*it).winInfo->setName(title.toUtf8().constData());
+    for (auto it = m_outputs.constBegin(); it != m_outputs.constEnd(); ++it) {
+        (*it)->setWindowTitle(title);
     }
 }
 
 void X11WindowedBackend::handleClientMessage(xcb_client_message_event_t *event)
 {
-    auto it = std::find_if(m_windows.begin(), m_windows.end(), [event] (const Output &o) { return o.window == event->window; });
-    if (it == m_windows.end()) {
+    auto it = std::find_if(m_outputs.begin(), m_outputs.end(),
+                           [event] (X11WindowedOutput *output) { return output->window() == event->window; }
+              );
+    if (it == m_outputs.end()) {
         return;
     }
     if (event->type == m_protocols && m_protocols != XCB_ATOM_NONE) {
         if (event->data.data32[0] == m_deleteWindowProtocol && m_deleteWindowProtocol != XCB_ATOM_NONE) {
-            if (m_windows.count() == 1) {
+            if (m_outputs.count() == 1) {
                 qCDebug(KWIN_X11WINDOWED) << "Backend window is going to be closed, shutting down.";
                 QCoreApplication::quit();
             } else {
                 // remove the window
                 qCDebug(KWIN_X11WINDOWED) << "Removing one output window.";
-                auto o = *it;
-                it = m_windows.erase(it);
-                xcb_unmap_window(m_connection, o.window);
-                xcb_destroy_window(m_connection, o.window);
-                delete o.winInfo;
+
+                auto removedOutput = *it;
+                it = m_outputs.erase(it);
 
                 // update the sizes
-                int x = o.internalPosition.x();
-                for (; it != m_windows.end(); ++it) {
-                    (*it).internalPosition.setX(x);
-                    x += (*it).size.width();
+                int x = removedOutput->internalPosition().x();
+                for (; it != m_outputs.end(); ++it) {
+                    (*it)->setGeometry(QPoint(x, 0), (*it)->pixelSize());
+                    x += (*it)->geometry().width();
                 }
+
+                delete removedOutput;
                 QMetaObject::invokeMethod(screens(), "updateCount");
             }
         }
@@ -334,8 +394,8 @@ void X11WindowedBackend::handleClientMessage(xcb_client_message_event_t *event)
 
 void X11WindowedBackend::handleButtonPress(xcb_button_press_event_t *event)
 {
-    auto it = std::find_if(m_windows.constBegin(), m_windows.constEnd(), [event] (const Output &o) { return o.window == event->event; });
-    if (it == m_windows.constEnd()) {
+    const X11WindowedOutput *output = findOutput(event->event);
+    if (!output) {
         return;
     }
     bool const pressed = (event->response_type & ~0x80) == XCB_BUTTON_PRESS;
@@ -347,9 +407,9 @@ void X11WindowedBackend::handleButtonPress(xcb_button_press_event_t *event)
         const int delta = (event->detail == XCB_BUTTON_INDEX_4 || event->detail == 6) ? -1 : 1;
         static const qreal s_defaultAxisStepDistance = 10.0;
         if (event->detail > 5) {
-            pointerAxisHorizontal(delta * s_defaultAxisStepDistance, event->time);
+            pointerAxisHorizontal(delta * s_defaultAxisStepDistance, event->time, delta);
         } else {
-            pointerAxisVertical(delta * s_defaultAxisStepDistance, event->time);
+            pointerAxisVertical(delta * s_defaultAxisStepDistance, event->time, delta);
         }
         return;
     }
@@ -369,9 +429,9 @@ void X11WindowedBackend::handleButtonPress(xcb_button_press_event_t *event)
         return;
     }
 
-    pointerMotion(QPointF(event->root_x - (*it).xPosition.x() + (*it).internalPosition.x(),
-                            event->root_y - (*it).xPosition.y() + (*it).internalPosition.y()) / it->scale,
-                            event->time);
+    const QPointF position = output->mapFromGlobal(QPointF(event->root_x, event->root_y));
+    pointerMotion(position, event->time);
+
     if (pressed) {
         pointerButtonPressed(button, event->time);
     } else {
@@ -386,22 +446,18 @@ void X11WindowedBackend::handleExpose(xcb_expose_event_t *event)
 
 void X11WindowedBackend::updateSize(xcb_configure_notify_event_t *event)
 {
-    auto it = std::find_if(m_windows.begin(), m_windows.end(), [event] (const Output &o) { return o.window == event->window; });
-    if (it == m_windows.end()) {
+    X11WindowedOutput *output = findOutput(event->window);
+    if (!output) {
         return;
     }
-    (*it).xPosition = QPoint(event->x, event->y);
-    QSize s = QSize(event->width, event->height);
-    if (s != (*it).size) {
-        (*it).size = s;
-        int x = (*it).internalPosition.x() + (*it).size.width() / (*it).scale;
-        it++;
-        for (; it != m_windows.end(); ++it) {
-            (*it).internalPosition.setX(x);
-            x += (*it).size.width() / (*it).scale;
-        }
-        emit sizeChanged();
+
+    output->setHostPosition(QPoint(event->x, event->y));
+
+    const QSize s = QSize(event->width, event->height);
+    if (s != output->pixelSize()) {
+        output->setGeometry(output->internalPosition(), s);
     }
+    emit sizeChanged();
 }
 
 void X11WindowedBackend::createCursor(const QImage &srcImage, const QPoint &hotspot)
@@ -418,12 +474,12 @@ void X11WindowedBackend::createCursor(const QImage &srcImage, const QPoint &hots
     xcb_create_pixmap(m_connection, 32, pix, m_screen->root, img.width(), img.height());
     xcb_create_gc(m_connection, gc, pix, 0, nullptr);
 
-    xcb_put_image(m_connection, XCB_IMAGE_FORMAT_Z_PIXMAP, pix, gc, img.width(), img.height(), 0, 0, 0, 32, img.byteCount(), img.constBits());
+    xcb_put_image(m_connection, XCB_IMAGE_FORMAT_Z_PIXMAP, pix, gc, img.width(), img.height(), 0, 0, 0, 32, img.sizeInBytes(), img.constBits());
 
     XRenderPicture pic(pix, 32);
     xcb_render_create_cursor(m_connection, cid, pic, qRound(hotspot.x() * outputScale), qRound(hotspot.y() * outputScale));
-    for (auto it = m_windows.constBegin(); it != m_windows.constEnd(); ++it) {
-        xcb_change_window_attributes(m_connection, (*it).window, XCB_CW_CURSOR, &cid);
+    for (auto it = m_outputs.constBegin(); it != m_outputs.constEnd(); ++it) {
+        xcb_change_window_attributes(m_connection, (*it)->window(), XCB_CW_CURSOR, &cid);
     }
 
     xcb_free_pixmap(m_connection, pix);
@@ -446,7 +502,7 @@ xcb_window_t X11WindowedBackend::rootWindow() const
 
 Screens *X11WindowedBackend::createScreens(QObject *parent)
 {
-    return new BasicScreens(this, parent);
+    return new OutputScreens(this, parent);
 }
 
 OpenGLBackend *X11WindowedBackend::createOpenGLBackend()
@@ -461,35 +517,27 @@ QPainterBackend *X11WindowedBackend::createQPainterBackend()
 
 void X11WindowedBackend::warpPointer(const QPointF &globalPos)
 {
-    const xcb_window_t w = m_windows.at(0).window;
+    const xcb_window_t w = m_outputs.at(0)->window();
     xcb_warp_pointer(m_connection, w, w, 0, 0, 0, 0, globalPos.x(), globalPos.y());
     xcb_flush(m_connection);
 }
 
 xcb_window_t X11WindowedBackend::windowForScreen(int screen) const
 {
-    if (screen > m_windows.count()) {
+    if (screen > m_outputs.count()) {
         return XCB_WINDOW_NONE;
     }
-    return m_windows.at(screen).window;
+    return m_outputs.at(screen)->window();
 }
 
-QVector<QRect> X11WindowedBackend::screenGeometries() const
+Outputs X11WindowedBackend::outputs() const
 {
-    QVector<QRect> ret;
-    for (auto it = m_windows.constBegin(); it != m_windows.constEnd(); ++it) {
-        ret << QRect((*it).internalPosition, (*it).size / (*it).scale);
-    }
-    return ret;
+    return m_outputs;
 }
 
-QVector<qreal> X11WindowedBackend::screenScales() const
+Outputs X11WindowedBackend::enabledOutputs() const
 {
-    QVector<qreal> ret;
-    for (auto it = m_windows.constBegin(); it != m_windows.constEnd(); ++it) {
-        ret << (*it).scale;
-    }
-    return ret;
+    return m_outputs;
 }
 
 }
