@@ -31,7 +31,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 // KWin
 #include "gestures.h"
-#include <client.h>
+#include <x11client.h>
 #include "cursor.h"
 #include "main.h"
 #include "platform.h"
@@ -538,7 +538,7 @@ void Edge::checkBlocking()
     }
     bool newValue = false;
     if (AbstractClient *client = Workspace::self()->activeClient()) {
-        newValue = client->isFullScreen() && client->geometry().contains(m_geometry.center());
+        newValue = client->isFullScreen() && client->frameGeometry().contains(m_geometry.center());
     }
     if (newValue == m_blocked) {
         return;
@@ -716,7 +716,7 @@ ScreenEdges::ScreenEdges(QObject *parent)
     , m_desktopSwitchingMovingClients(false)
     , m_timeThreshold(0)
     , m_reactivateThreshold(0)
-    , m_virtualDesktopLayout(nullptr)
+    , m_virtualDesktopLayout({})
     , m_actionTopLeft(ElectricActionNone)
     , m_actionTop(ElectricActionNone)
     , m_actionTopRight(ElectricActionNone)
@@ -880,7 +880,7 @@ void ScreenEdges::setActionForTouchBorder(ElectricBorder border, ElectricBorderA
 void ScreenEdges::updateLayout()
 {
     const QSize desktopMatrix = VirtualDesktopManager::self()->grid().size();
-    Qt::Orientations newLayout = nullptr;
+    Qt::Orientations newLayout = {};
     if (desktopMatrix.width() > 1) {
         newLayout |= Qt::Horizontal;
     }
@@ -1262,7 +1262,7 @@ void ScreenEdges::createEdgeForClient(AbstractClient *client, ElectricBorder bor
     int x = 0;
     int width = 0;
     int height = 0;
-    const QRect geo = client->geometry();
+    const QRect geo = client->frameGeometry();
     const QRect fullArea = workspace()->clientArea(FullArea, 0, 1);
     for (int i = 0; i < screens()->count(); ++i) {
         const QRect screen = screens()->geometry(i);
