@@ -29,7 +29,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * @author Martin Gräßlin <mgraesslin@kde.org>
  * @since 4.4
- **/
+ */
 
 namespace KWin
 {
@@ -42,7 +42,7 @@ namespace TabBox
  *
  * @author Martin Gräßlin <mgraesslin@kde.org>
  * @since 4.4
- **/
+ */
 class ClientModel
     : public QAbstractItemModel
 {
@@ -58,19 +58,20 @@ public:
         CloseableRole = Qt::UserRole + 7 ///< TabBoxClient can be closed
     };
     explicit ClientModel(QObject* parent = nullptr);
-    ~ClientModel();
-    virtual QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
-    virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
-    virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
-    virtual QModelIndex parent(const QModelIndex& child) const;
-    virtual QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const;
+    ~ClientModel() override;
+    QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const override;
+    int columnCount(const QModelIndex& parent = QModelIndex()) const override;
+    int rowCount(const QModelIndex& parent = QModelIndex()) const override;
+    QModelIndex parent(const QModelIndex& child) const override;
+    QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const override;
+    QHash<int, QByteArray> roleNames() const override;
     Q_INVOKABLE QString longestCaption() const;
 
     /**
      * @param client The TabBoxClient whose index should be returned
      * @return Returns the ModelIndex of given TabBoxClient or an invalid ModelIndex
      * if the model does not contain the given TabBoxClient.
-     **/
+     */
     QModelIndex index(QWeakPointer<TabBoxClient> client) const;
 
     /**
@@ -81,16 +82,16 @@ public:
      * list.
      * @param desktop The desktop for which the list should be created
      * @param partialReset Keep the currently selected client or regenerate everything
-     **/
+     */
     void createClientList(int desktop, bool partialReset = false);
     /**
      * This method is provided as a overload for current desktop
      * @see createClientList
-     **/
+     */
     void createClientList(bool partialReset = false);
     /**
      * @return Returns the current list of TabBoxClients.
-     **/
+     */
     TabBoxClientList clientList() const {
         return m_clientList;
     }
@@ -100,7 +101,7 @@ public Q_SLOTS:
     /**
      * Activates the client at @p index and closes the TabBox.
      * @param index The row index
-     **/
+     */
     void activate(int index);
 
 private:
