@@ -23,7 +23,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "activities.h"
 #endif
 #include "atoms.h"
-#include "client.h"
+#include "x11client.h"
 #include "client_machine.h"
 #include "composite.h"
 #include "effects.h"
@@ -43,7 +43,7 @@ Toplevel::Toplevel()
     : m_visual(XCB_NONE)
     , bit_depth(24)
     , info(nullptr)
-    , ready_for_painting(true)
+    , ready_for_painting(false)
     , m_isDamaged(false)
     , m_internalId(QUuid::createUuid())
     , m_client()
@@ -76,22 +76,6 @@ QDebug& operator<<(QDebug& stream, const Toplevel* cl)
     return stream;
 }
 
-QDebug& operator<<(QDebug& stream, const ToplevelList& list)
-{
-    stream << "LIST:(";
-    bool first = true;
-    for (ToplevelList::ConstIterator it = list.begin();
-            it != list.end();
-            ++it) {
-        if (!first)
-            stream << ":";
-        first = false;
-        stream << *it;
-    }
-    stream << ")";
-    return stream;
-}
-
 QRect Toplevel::decorationRect() const
 {
     return rect();
@@ -110,7 +94,7 @@ void Toplevel::detectShape(xcb_window_t id)
 void Toplevel::copyToDeleted(Toplevel* c)
 {
     m_internalId = c->internalId();
-    geom = c->geom;
+    m_frameGeometry = c->m_frameGeometry;
     m_visual = c->m_visual;
     bit_depth = c->bit_depth;
     info = c->info;
@@ -133,6 +117,7 @@ void Toplevel::copyToDeleted(Toplevel* c)
     m_screen = c->m_screen;
     m_skipCloseAnimation = c->m_skipCloseAnimation;
     m_internalFBO = c->m_internalFBO;
+    m_internalImage = c->m_internalImage;
 }
 
 // before being deleted, remove references to everything that's now
@@ -144,11 +129,14 @@ void Toplevel::disownDataPassedToDeleted()
 
 QRect Toplevel::visibleRect() const
 {
-    QRect r = decorationRect();
-    if (hasShadow() && !shadow()->shadowRegion().isEmpty()) {
-        r |= shadow()->shadowRegion().boundingRect();
+    // There's no strict order between frame geometry and buffer geometry.
+    QRect rect = frameGeometry() | bufferGeometry();
+
+    if (shadow() && !shadow()->shadowRegion().isEmpty()) {
+        rect |= shadow()->shadowRegion().boundingRect().translated(pos());
     }
-    return r.translated(geometry().topLeft());
+
+    return rect;
 }
 
 Xcb::Property Toplevel::fetchWmClientLeader() const
@@ -240,6 +228,11 @@ void Toplevel::setResourceClass(const QByteArray &name, const QByteArray &classN
     emit windowClassChanged();
 }
 
+bool Toplevel::resourceMatch(const Toplevel *c1, const Toplevel *c2)
+{
+    return c1->resourceClass() == c2->resourceClass();
+}
+
 double Toplevel::opacity() const
 {
     if (info->opacity() == 0xffffffff)
@@ -326,7 +319,7 @@ bool Toplevel::compositing() const
     return Workspace::self()->compositing();
 }
 
-void Client::damageNotifyEvent()
+void X11Client::damageNotifyEvent()
 {
     if (syncRequest.isPending && isResize()) {
         emit damaged(this, QRect());
@@ -404,8 +397,11 @@ void Toplevel::getDamageRegionReply()
         region += QRect(reply->extents.x, reply->extents.y,
                         reply->extents.width, reply->extents.height);
 
+    const QRect bufferRect = bufferGeometry();
+    const QRect frameRect = frameGeometry();
+
     damage_region += region;
-    repaints_region += region;
+    repaints_region += region.translated(bufferRect.topLeft() - frameRect.topLeft());
 
     free(reply);
 }
@@ -415,10 +411,18 @@ void Toplevel::addDamageFull()
     if (!compositing())
         return;
 
-    damage_region = rect();
-    repaints_region |= rect();
+    const QRect bufferRect = bufferGeometry();
+    const QRect frameRect = frameGeometry();
 
-    emit damaged(this, rect());
+    const int offsetX = bufferRect.x() - frameRect.x();
+    const int offsetY = bufferRect.y() - frameRect.y();
+
+    const QRect damagedRect = QRect(0, 0, bufferRect.width(), bufferRect.height());
+
+    damage_region = damagedRect;
+    repaints_region |= damagedRect.translated(offsetX, offsetY);
+
+    emit damaged(this, damagedRect);
 }
 
 void Toplevel::resetDamage()
@@ -522,7 +526,7 @@ void Toplevel::checkScreen()
             emit screenChanged();
         }
     } else {
-        const int s = screens()->number(geometry().center());
+        const int s = screens()->number(frameGeometry().center());
         if (s != m_screen) {
             m_screen = s;
             emit screenChanged();
@@ -558,9 +562,14 @@ qreal Toplevel::screenScale() const
     return m_screenScale;
 }
 
+qreal Toplevel::bufferScale() const
+{
+    return surface() ? surface()->scale() : 1;
+}
+
 bool Toplevel::isOnScreen(int screen) const
 {
-    return screens()->geometry(screen).intersects(geometry());
+    return screens()->geometry(screen).intersects(frameGeometry());
 }
 
 bool Toplevel::isOnActiveScreen() const
@@ -568,11 +577,11 @@ bool Toplevel::isOnActiveScreen() const
     return isOnScreen(screens()->current());
 }
 
-void Toplevel::getShadow()
+void Toplevel::updateShadow()
 {
     QRect dirtyRect;  // old & new shadow region
     const QRect oldVisibleRect = visibleRect();
-    if (hasShadow()) {
+    if (shadow()) {
         dirtyRect = shadow()->shadowRegion().boundingRect();
         if (!effectWindow()->sceneWindow()->shadow()->updateShadow()) {
             effectWindow()->sceneWindow()->updateShadow(nullptr);
@@ -581,7 +590,7 @@ void Toplevel::getShadow()
     } else {
         Shadow::createShadow(this);
     }
-    if (hasShadow())
+    if (shadow())
         dirtyRect |= shadow()->shadowRegion().boundingRect();
     if (oldVisibleRect != visibleRect())
         emit paddingChanged(this, oldVisibleRect);
@@ -591,14 +600,6 @@ void Toplevel::getShadow()
     }
 }
 
-bool Toplevel::hasShadow() const
-{
-    if (effectWindow() && effectWindow()->sceneWindow()) {
-        return effectWindow()->sceneWindow()->shadow() != nullptr;
-    }
-    return false;
-}
-
 Shadow *Toplevel::shadow()
 {
     if (effectWindow() && effectWindow()->sceneWindow()) {
@@ -770,15 +771,6 @@ QRegion Toplevel::inputShape() const
     }
 }
 
-void Toplevel::setInternalFramebufferObject(const QSharedPointer<QOpenGLFramebufferObject> &fbo)
-{
-    if (m_internalFBO != fbo) {
-        discardWindowPixmap();
-        m_internalFBO = fbo;
-    }
-    setDepth(32);
-}
-
 QMatrix4x4 Toplevel::inputTransformation() const
 {
     QMatrix4x4 m;
@@ -793,7 +785,7 @@ quint32 Toplevel::windowId() const
 
 QRect Toplevel::inputGeometry() const
 {
-    return geometry();
+    return frameGeometry();
 }
 
 bool Toplevel::isLocalhost() const
@@ -804,5 +796,15 @@ bool Toplevel::isLocalhost() const
     return m_clientMachine->isLocal();
 }
 
+QMargins Toplevel::bufferMargins() const
+{
+    return QMargins();
+}
+
+QMargins Toplevel::frameMargins() const
+{
+    return QMargins();
+}
+
 } // namespace
 
